@@ -143,6 +143,7 @@ class Session(BaseModel):
     ports: Optional[Tuple[int, ...]] = None
     priority: Optional[Priority] = None
     work_dir: Optional[str] = None
+    identity: Optional[str] = None
 
 
 class Job(BaseModel):
@@ -200,6 +201,7 @@ class Job(BaseModel):
 class Jobs(BaseModel):
     data: Optional[Tuple[Job, ...]] = None
     next: Optional[str] = None
+    next_cursor: Optional[str] = None
 
 
 class JobStatusUpdate(BaseModel):
